package es.verdnatura.presentation.view.feature.login.fragment

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Transformations
import com.google.gson.Gson
import com.google.gson.JsonObject
import com.google.gson.reflect.TypeToken
import es.verdnatura.domain.SilexCallback
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.base.nameofFunction
import es.verdnatura.presentation.common.Event
import es.verdnatura.presentation.common.ResponseItemNumber
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.ajustes.model.Printers
import es.verdnatura.presentation.view.feature.ajustes.model.SectorItemVO
import es.verdnatura.presentation.view.feature.ajustes.model.WorkerPrintersList
import es.verdnatura.presentation.view.feature.historicoshelvinglog.model.DeviceLogSalix
import es.verdnatura.presentation.view.feature.login.model.DataUserSalix
import es.verdnatura.presentation.view.feature.login.model.LoginDevice
import es.verdnatura.presentation.view.feature.login.model.LoginSalixVO
import es.verdnatura.presentation.view.feature.login.model.OperatorList
import es.verdnatura.presentation.view.feature.login.model.OperatorSalix
import es.verdnatura.presentation.view.feature.login.model.RenewToken
import es.verdnatura.presentation.view.feature.login.model.accessConfigSalix
import es.verdnatura.presentation.view.feature.login.model.accessConfigSalixList
import es.verdnatura.presentation.view.feature.login.model.versionApp
import es.verdnatura.presentation.view.feature.login.model.workerId
import es.verdnatura.presentation.view.feature.paletizador.model.itemsExpeditionDynamics
import retrofit2.Response
import java.util.Date

class LoginViewModel(val context: Context) : BaseViewModel(context) {
    private val _loginsalixitem by lazy { MutableLiveData<LoginSalixVO>() }
    val loginsalixitem: LiveData<LoginSalixVO>
        get() = _loginsalixitem


    private val _devicelogresponse by lazy { MutableLiveData<ResponseItemVO>() }
    val devicelogresponse: LiveData<ResponseItemVO>
        get() = _devicelogresponse

    private val _operatorAdd by lazy { MutableLiveData<ResponseItemVO>() }
    val operatorAdd: LiveData<ResponseItemVO>
        get() = _operatorAdd

    private val _versionappitem by lazy { MutableLiveData<versionApp>() }
    val versionappitem: LiveData<versionApp>
        get() = _versionappitem

    private val _logindevice by lazy { MutableLiveData<LoginDevice>() }
    val logindevice: LiveData<LoginDevice>
        get() = _logindevice

    private val _workerId by lazy { MutableLiveData<workerId>() }
    val workerId: LiveData<workerId>
        get() = _workerId


    private val _workergetSector by lazy { MutableLiveData<SectorItemVO>() }
    val workergetSector: LiveData<SectorItemVO>
        get() = _workergetSector

    private val _workergetPrinter by lazy { MutableLiveData<Printers>() }
    val workergetPrinter: LiveData<Printers>
        get() = _workergetPrinter

    private val _operatorGetNumberOfWagons by lazy { MutableLiveData<ResponseItemNumber>() }
    val operatorGetNumberOfWagons: LiveData<ResponseItemNumber>
        get() = _operatorGetNumberOfWagons


    private val _workerPrintList by lazy { MutableLiveData<WorkerPrintersList>() }
    private val _workerOperatorList by lazy { MutableLiveData<OperatorList>() }
    private val _accessConfigSalixList by lazy { MutableLiveData<accessConfigSalixList>() }

    private val _renewTokenResponse by lazy { MutableLiveData<RenewToken>() }
    val renewTokenResponse: LiveData<RenewToken>
        get() = _renewTokenResponse

    val loadWorkerPrintList = Transformations.map(_workerPrintList) { Event(it) }
    val loadOperatorList = Transformations.map(_workerOperatorList) { Event(it) }
    val loadAccessConfigSalixList = Transformations.map(_accessConfigSalixList) { Event(it) }

    fun loginSalix(user: String, password: String) {
        salix.login(LoginSalixVO(user, password)).enqueue(
            object : SilexCallback<LoginSalixVO>(context) {
                override fun onSuccess(response: Response<LoginSalixVO>) {

                    val loginSalixVO: LoginSalixVO?
                    if (response.body() != null) {
                        loginSalixVO = response.body()?.let {
                            LoginSalixVO(
                                user,
                                password,
                                it.token,
                                it.created,
                                it.ttl,
                                isError = false,
                                errorMessage = ""
                            )
                        }
                    } else {

                        loginSalixVO = LoginSalixVO(
                            user,
                            password,
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this), response.message()
                            )
                        )


                    }

                    _loginsalixitem.value = loginSalixVO
                }

                override fun onError(t: Throwable) {
                    val loginSalixVO =
                        LoginSalixVO(
                            user,
                            password,
                            "",
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                t.message!!
                            )
                        )
                    _loginsalixitem.value = loginSalixVO
                }
            })
    }

    fun device_checkLogin(android_id: String) {

        silex.device_checkLogin(android_id)
            .enqueue(object : SilexCallback<LoginDevice>(context) {
                override fun onSuccess(response: Response<LoginDevice>) {
                    if (response.body() != null) {

                        response.body()?.let {

                            _logindevice.value = LoginDevice(
                                it.vMessage, it.vIsAuthorized,
                                isError = false,
                                errorMessage = ""
                            )

                        }
                    } else {
                        _logindevice.value = LoginDevice(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            )
                        )
                    }

                }

                override fun onError(t: Throwable) {
                    _logindevice.value =
                        LoginDevice(
                            "",
                            "",
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                t.message!!
                            )
                        )
                }
            })
    }

    fun getCurrentUserData() {
        salix.getCurrentUserData().enqueue(object : SilexCallback<DataUserSalix>(context) {
            override fun onSuccess(response: Response<DataUserSalix>) {
                app.userName = response.body()?.let { it.nickname }
                app.userId = response.body()?.let { it.id }
            }

        })
    }

    fun deviceLog_add(
        app: String, versionApp: String, android_id: String
    ) {
        //Tarea 4815
        // salix.deviceLogInsert(DeviceLogSalix(androidId = android_id,userFk,nameApp= app,versionApp= versionApp))
        silex.deviceLog_add(app, versionApp, android_id)
            .enqueue(object : SilexCallback<Void>(context) {

                override fun onSuccess(response: Response<Void>) {
                }

            })

    }
    fun deviceLog_addSalix(
        app: String, versionApp: String, android_id: String,userFk:Int, serialnumber:String
    ) {

        //Tarea 4815
         salix.deviceLogInsert(DeviceLogSalix(android_id = android_id,userFk=userFk,nameApp= app,versionApp= versionApp,serialNumber = serialnumber))
            .enqueue(object : SilexCallback<Void>(context) {

                override fun onSuccess(response: Response<Void>) {
                }

            })

    }

    fun checkVersion(nameApp: String) {
        silex.getVersion(nameApp)
            .enqueue(object : SilexCallback<versionApp>(context) {
                override fun onSuccess(response: Response<versionApp>) {
                    if (response.body() != null) {
                        _versionappitem.value = response.body()?.let {
                            versionApp(
                                it.appname,
                                it.version,
                                it.IsVersionCritical,
                                it.url,
                                isError = false,
                                errorMessage = ""
                            )

                        }

                    } else {
                        _versionappitem.value = versionApp(
                            "",
                            "",
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                response.message()
                            )
                        )
                    }
                }

                override fun onError(t: Throwable) {
                    _versionappitem.value =
                        versionApp(
                            nameApp,
                            "",
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                t.message!!
                            )
                        )
                }
            })
    }

    /* fun nodeJsTest(smarttag: String, wagon: String, shelving: String)
     {
         getLoginUserCase.nodeJsTest(smarttag, wagon, shelving)
             .enqueue(object : Callback<NodeJsService.SmartTagNode>
             {
                 override fun onResponse(
                     call: Call<NodeJsService.SmartTagNode>,
                     response: Response<NodeJsService.SmartTagNode>
                                        )
                 {


                     val workerId: NodeJsService.SmartTagNode?

                     *//*   if (response.isSuccessful()){
                           //Log.i("VERDNATURA::","OK")
                       }

                       if (!response.errorBody().toString().isEmpty()){

                       }*//*

                    if (response.body() != null)
                    {

                        workerId = NodeJsService.SmartTagNode(
                            response.body().toString(),
                            isError = false,
                            errorMessage = ""
                                                             )

                    } else
                    {
                        workerId = NodeJsService.SmartTagNode(
                            isError = true,
                            errorMessage = "Error al llamar al servicio Node."
                                                             )
                    }

                    _nodejs.value = workerId
                }

                override fun onFailure(call: Call<NodeJsService.SmartTagNode>, t: Throwable)
                {
                    val workerId =
                        NodeJsService.SmartTagNode(isError = true, errorMessage = t.message!!)

                      //Log.i("VERDNATURA::","el error nodejs es "+t.message+"--"+t.cause.toString()+"--"+t.stackTrace.toString())

                    _nodejs.value = workerId
                }

            })
    }*/

    fun getListFromJSON(json: JsonObject): MutableList<itemsExpeditionDynamics> {
        val gson = Gson()
        var list = mutableListOf<itemsExpeditionDynamics>()

        //  val mapType = object : TypeToken<Map<String, Any>>() {}.type

        var expeditionState: Map<String, Any> =
            gson.fromJson(json, object : TypeToken<Map<String, Any>>() {}.type)
        expeditionState.forEach {

            list.add(itemsExpeditionDynamics(key = it.key, value = it.value.toString()))

        }
        return list
    }

    fun worker_getSector() {

        silex.worker_getSector().enqueue(object : SilexCallback<SectorItemVO>(context) {
            override fun onError(t: Throwable) {
                _workergetSector.value = SectorItemVO(
                    0,
                    "",
                    0,
                    isError = true,
                    errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                )
            }

            override fun onSuccess(response: Response<SectorItemVO>) {
                _workergetSector.value = response.body()?.let {
                    SectorItemVO(
                        it.id,
                        it.description,
                        it.warehouseFk,
                        isError = false
                    )
                }

            }

        })
    }

    fun operator_getNumberOfWagons() {

        silex.operator_getNumberOfWagons()
            .enqueue(object : SilexCallback<Int?>(context) {
                override fun onError(t: Throwable) {
                    _operatorGetNumberOfWagons.value = ResponseItemNumber(
                        null,
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                    )
                }

                override fun onSuccess(response: Response<Int?>) {
                    if (response.body() != null)
                        _operatorGetNumberOfWagons.value =
                            ResponseItemNumber(response.body(), isError = false)


                }
            })
    }

    fun operator_add() {
       //Tarea 5134 no se puede modificar directamente con un upsertWith, hay que modificar front. Y si al hacer login en Salix hacer el insertIgnore en -> operator_add, back de Salix sabe
        //si es de app?
        //salix.operatorAdd()
        silex.operator_add().enqueue(object : SilexCallback<Any>(context) {
            override fun onError(t: Throwable) {
                _operatorAdd.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(nameofFunction(this), t.message!!)
                )
            }

            override fun onSuccess(response: Response<Any>) {
                _operatorAdd.value = ResponseItemVO("", false)
            }
        })
    }

    fun worker_getPrinter() {

        silex.worker_getPrinter()
            .enqueue(object : SilexCallback<List<Printers>>(context) {
                override fun onError(t: Throwable) {
                    val listError: ArrayList<Printers> = ArrayList()
                    listError.add(
                        Printers(0,"",
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                t.message!!
                            )
                        )
                    )
                    _workerPrintList.value = WorkerPrintersList(listError)

                }

                override fun onSuccess(response: Response<List<Printers>>) {
                    if (response.body() != null) {
                        _workerPrintList.value = response.body()?.let {
                            WorkerPrintersList(it)
                        }

                    } else {
                        val listError: ArrayList<Printers> = ArrayList()
                        listError.add(
                            Printers(0,"",
                                isError = true,
                                errorMessage = getMessageFromAllResponse(
                                    nameofFunction(this),
                                    response.message()
                                )
                            )
                        )
                        _workerPrintList.value = WorkerPrintersList(listError)

                    }

                }

            })
    }
    fun operator_getDataSalix(
        workerId: String
    ) {

        salix.operator_getData(workerId)
            .enqueue(object : SilexCallback<List<OperatorSalix>>(context) {
                override fun onError(t: Throwable) {
                    val listError: ArrayList<OperatorSalix> = ArrayList()
                    listError.add(
                        OperatorSalix(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                t.message!!
                            )
                        )
                    )
                    _workerOperatorList.value = OperatorList(listError)

                }

                override fun onSuccess(response: Response<List<OperatorSalix>>) {
                    if (response.body() != null) {
                        _workerOperatorList.value = response.body()?.let {
                            OperatorList(it)
                        }

                    } else {
                        val listError: ArrayList<OperatorSalix> = ArrayList()
                        listError.add(
                            OperatorSalix(
                                isError = true,
                                errorMessage = getMessageFromAllResponse(
                                    nameofFunction(this),
                                    response.message()
                                )
                            )
                        )
                        _workerOperatorList.value = OperatorList(listError)

                    }

                }

            })
    }
    fun getAccessTokenConfigs(
    ) {

        salix.getAccessTokenConfigs()
            .enqueue(object : SilexCallback<List<accessConfigSalix>>(context) {
                override fun onError(t: Throwable) {
                    val listError: ArrayList<accessConfigSalix> = ArrayList()
                    listError.add(
                        accessConfigSalix(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(
                                nameofFunction(this),
                                t.message!!
                            )
                        )
                    )
                    _accessConfigSalixList.value = accessConfigSalixList(listError)

                }

                override fun onSuccess(response: Response<List<accessConfigSalix>>) {
                    if (response.body() != null) {
                        _accessConfigSalixList.value = response.body()?.let {
                            accessConfigSalixList(it)
                        }

                    } else {
                        val listError: ArrayList<accessConfigSalix> = ArrayList()
                        listError.add(
                            accessConfigSalix(
                                isError = true,
                                errorMessage = getMessageFromAllResponse(
                                    nameofFunction(this),
                                    response.message()
                                )
                            )
                        )
                        _accessConfigSalixList.value = accessConfigSalixList(listError)

                    }

                }

            })
    }
    fun renewToken(
    ) {
        salix.renewToken()
            .enqueue(object : SilexCallback<RenewToken>(context) {

                override fun onSuccess(response: Response<RenewToken>) {

                    var RenewToken = response.body()
                    val prefs = app.getPrefsEditor()
                    prefs.putString("token", RenewToken!!.id)
                    prefs.putLong("ttl", RenewToken!!.ttl)
                    prefs.putLong("tokenCreated", Date().time)
                    _renewTokenResponse.value = RenewToken

                }
                override fun onError(t: Throwable) {

                    _renewTokenResponse.value = RenewToken(
                        "", 0, true, getMessageFromAllResponse(
                            nameofFunction(this),
                            t.message!!
                        )
                    )


                }

            })
    }
}
