package es.verdnatura.presentation.view.feature.presacador.fragment

import android.content.Context
import android.graphics.drawable.Drawable
import android.os.Build
import android.text.InputType
import android.view.View
import android.view.inputmethod.EditorInfo
import android.widget.ImageView
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentSacadorBinding
import es.verdnatura.domain.ConstAndValues.PREITEMPICKER
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnCollectionSelectedListener
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.ToolBarAdapterTooltip
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.feature.sacador.adapter.CollectionAdapter
import es.verdnatura.presentation.view.feature.sacador.model.CollectionVO


class InitPreSacadorFragment(
    var entrypoint: String = ""
) :
    BaseFragment<FragmentSacadorBinding, PreSacadorViewModel>(PreSacadorViewModel::class) {

    private var onCollectionSelectedListener: OnCollectionSelectedListener? = null
    private var goBack: Boolean = false
    private var goBack2: Boolean = false
    private lateinit var customDialog: CustomDialog
    private var collectionsList: ArrayList<CollectionVO> = ArrayList()
    private var adapter: CollectionAdapter? = null
    private lateinit var customDialogInput: CustomDialogInput


    companion object {
        fun newInstance(entryPoint: String) = InitPreSacadorFragment(entryPoint)
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        if (context is OnCollectionSelectedListener) onCollectionSelectedListener = context
    }

    override fun getLayoutId(): Int = R.layout.fragment_sacador

    override fun init() {

        customDialog = CustomDialog(requireContext())
        customDialogInput = CustomDialogInput(requireContext())
        binding.splashProgress.visibility = View.VISIBLE
        binding.splashProgressTwo.visibility = View.GONE
        setToolBar()
        setEvents()
        viewModel.sectorCollection_get()
        super.init()

    }

    override fun onPause() {
        goBack = true
        goBack2 = true
        super.onPause()
    }

    private fun setToolBar() {

        ma.hideBottomNavigation(View.VISIBLE)
        binding.mainToolbar.toolbarTitle.text = entrypoint
        binding.mainToolbar.backButton.setOnClickListener {
            ma.onBackPressed()
        }

        val listIcons: ArrayList<ImageView> = ArrayList()
        val iconAdd = ImageView(context)
        val iconPrint = ImageView(context)
        iconPrint.setImageResource(R.drawable.ic_print_black_24dp)
        iconAdd.setImageResource(R.drawable.ic_add_black_24dp)


        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {

            iconAdd.tooltipText = getTooltip(R.drawable.ic_add_black_24dp)
            iconPrint.tooltipText = getTooltip(R.drawable.ic_print_black_24dp)

        }


        listIcons.add(iconAdd)
        //  Tarea  4731
        //listIcons.add(iconPrint)

        binding.mainToolbar.toolbarIcons.adapter =
            ToolBarAdapterTooltip(listIcons, object : OnOptionsSelectedListener {
                override fun onOptionsItemSelected(item: Drawable) {

                    when (item) {

                        iconAdd.drawable -> {
                            binding.splashProgress.visibility = View.VISIBLE
                            viewModel.sectorCollection_new(getDataInt(SECTORFK))
                        }
                        iconPrint.drawable -> {
                            showPrintPrevious()
                        }
                    }
                }
            })
        binding.mainToolbar.toolbarIcons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun print() {
        if (ma.havePrinter()) {
            binding.splashProgress.visibility = View.VISIBLE
            viewModel.ticket_printLabelPrevious(customDialogInput.getValue().toInt())
            (getString(R.string.Imprimiendo) + getData(PRINTERNAME)).toast(
                requireContext()
            )
        } else {
            getString(R.string.selectPrinter).toast(requireContext())
        }
    }

    private fun showPrintPrevious() {

        customDialogInput.setInputText(InputType.TYPE_CLASS_NUMBER)
        customDialogInput.setTitle(getString(R.string.printTicket))
        customDialogInput.setDescription(getString(R.string.printPreviousGroup))
            .setValue("")
            .setOkButton(getString(R.string.print)) {
                try {
                    if (customDialogInput.getValue().isNotEmpty()
                    ) {
                        print()
                    }
                } catch (ex: Exception) {
                    getString(R.string.errorInput)
                }
                customDialogInput.setValue("")

            }.setKoButton(getString(R.string.cancel)) {
                customDialogInput.dismiss()
            }.show()
        customDialogInput.setFocusText()


        customDialogInput.setFocusText()
        ma.hideKeyboard(customDialogInput.getEditText())

        customDialogInput.getEditText().setOnEditorActionListener { _, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5 || actionId == 6) {

                if (customDialogInput.getValue().isNotEmpty()
                ) {
                    print()
                }

                return@setOnEditorActionListener true
            }
            false
        }


    }

    private fun setEvents() {
        binding.sacadorSwipe.setOnRefreshListener {
            viewModel.sectorCollection_get()
            binding.sacadorSwipe.isRefreshing = false
        }
    }


    override fun observeViewModel() {

        with(viewModel) {

            collectionList.observe(viewLifecycleOwner, Observer {
                if (!goBack2) binding.splashProgress.visibility = View.GONE
                createCollectionList(it.list)
                binding.splashProgress.visibility = View.GONE
                goBack2 = false
            })

            response.observe(viewLifecycleOwner, Observer {
                binding.splashProgressTwo.visibility = View.GONE
                binding.splashProgress.visibility = View.GONE
                if (it.isError) {
                    ma.messageWithSound(it.errorMessage, true, false)

                } else {
                    if (goBack) addCollectionToList(it.response.toInt())
                }
                goBack = false
                //sergio:para asegurarnos que aparece en pantalla. En mi movil personal y trabajo no hacia falta.
                //viewModel.PrecollectionGetSalix(token=getData(TOKEN))
            })
            responseNew.observe(viewLifecycleOwner, Observer {
                binding.splashProgressTwo.visibility = View.GONE
                binding.splashProgress.visibility = View.GONE
                if (it.isError) {
                    ma.messageWithSound(
                        getString(R.string.errorCollectionNew) + it.errorMessage,
                        true,
                        false
                    )

                } else {
                    //if (goBack)
                    viewModel.sectorCollection_get()
                }
                goBack = false
                //sergio:para asegurarnos que aparece en pantalla. En mi movil personal y trabajo no hacia falta.
                //viewModel.PrecollectionGetSalix(token=getData(TOKEN))
            })
        }
    }

    private fun createCollectionList(collections: List<CollectionVO>) {
        collectionsList = ArrayList()
        collections.forEach {
            collectionsList.add(it)
        }
        val lm = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
        adapter = CollectionAdapter(collectionsList, onCollectionSelectedListener!!, PREITEMPICKER)
        binding.fragmentSacadorCollections.layoutManager = lm
        binding.fragmentSacadorCollections.adapter = adapter

    }

    private fun addCollectionToList(collection: Int) {
        var isNew = true
        collectionsList.forEach {
            if (it.collectionFk == collection) {
                isNew = false
            }
        }
        if (isNew) {
            collectionsList.add(
                CollectionVO(
                    collectionFk = collection,
                    created = getString(R.string.newCreated)
                )
            )
            adapter?.notifyDataSetChanged()
        } else {
            customDialog.setTitle(getString(R.string.info))
                .setDescription(getString(R.string.collectionPending) + collection)
                .setOkButton(getString(R.string.accept)) {
                    customDialog.dismiss()
                }.show()
        }
        //sergio:para asegurarnos que se actualiza pantalla.
        adapter?.notifyDataSetChanged()
    }


}
